@@ -1,8 +1,7 @@
-"""Module for various text/doc processing functions/classes."""
+"""Module for various doc/text processing classes/functions."""
 from __future__ import annotations
 
-from itertools import tee
-from typing import Any, Callable, Iterable
+from typing import Callable
 
 from rapidfuzz import fuzz
 from spacy.tokens import Doc
@@ -17,15 +16,6 @@ def map_chars_to_tokens(doc: Doc) -> dict[int, int]:
     return chars_to_tokens
 
 
-def n_wise(iterable: Iterable[Any], n: int) -> Iterable[Any]:
-    """Iterates over an iterables in slices of length n by one step at a time."""
-    iterables = tee(iterable, n)
-    for i in range(len(iterables)):
-        for _ in range(i):
-            next(iterables[i], None)
-    return zip(*iterables)
-
-
 class FuzzyFuncs:
     """Container class housing fuzzy matching functions.
 
@@ -53,7 +43,7 @@ class FuzzyFuncs:
     """
 
     def __init__(self: FuzzyFuncs, match_type: str = "phrase") -> None:
-        """Initializes a fuzzyfuncs container.
+        """Initializes a `FuzzyFuncs` container.
 
         Args:
             match_type: Whether the fuzzy matching functions
@@ -74,13 +64,13 @@ class FuzzyFuncs:
                 "partial_token_sort": fuzz.partial_token_sort_ratio,
                 "quick": fuzz.QRatio,
                 "weighted": fuzz.WRatio,
-                "quick_lev": fuzz.quick_lev_ratio,
+                "token": fuzz.token_ratio,
+                "partial_token": fuzz.partial_token_ratio,
             }
         elif match_type == "token":
             self._fuzzy_funcs = {
                 "simple": fuzz.ratio,
                 "quick": fuzz.QRatio,
-                "quick_lev": fuzz.quick_lev_ratio,
             }
         else:
             raise ValueError("match_type must be either 'phrase' or 'token'.")
